;invert
;-----
;by Levi Lansing
;--puzzlesome@hotmail.com
;--http://www.crosswinds.net/levisti86/

;this program was taken from Flash (by me) and just modified and extended
;which is why the two sources are similar

#include "asm86.h"
#include "ti86asm.inc"

_user_int_ram EQU $d2fd
_off  equ  4101h

.org _asm_exec_ram
  bit 2,(iy+$23)	;is the user interrupt on?
  jr nz,undo		;go turn it it off if it is
   res 2,(iy+$23)    ;turn user int off so it won't get called by
                     ;accident before we're ready
   
  call _clrScrn		;clear the screen
  call _homeup		;put the cursor at the top of the screen

   ld hl,int         		;hl point to the begining of the program
   ld de,_user_int_ram+1	;de pionts to where the program is to be loaded
   ld bc,int_end-int		;bc is the length of the program to be sent
   ldir						;copies it
   
   ld a,(_user_int_ram+1)		;this sets up checksum byte
   ld hl,_user_int_ram+($28*1)	;we need to add the 1st bite with one
   add a,(hl)					;every 40 bytes and store it in the...
   ld hl,_user_int_ram+($28*2)
   add a,(hl)
   ld hl,_user_int_ram+($28*3)
   add a,(hl)
   ld hl,_user_int_ram+($28*4)
   add a,(hl)
   ld hl,_user_int_ram+($28*5)
   add a,(hl)
   ld (_user_int_ram),a			;first byte of the interrupt area

   ld a,$36			;$36+$0C*$100 = F600 (the new video mem location)
   out (0),a		;make sure the calc's video mem is set to display this
   ld a,13			;load up the presets...
   ld (var),a		;var is the counter for the speed
   dec a			;a=12
   ld (Speed),a		;Speed of the refresh rate
   set 2,(iy+$23)	;turn on the interrupt
   ld hl,TEXT		;point hl at the text
   call _puts		;put the text on the screen
   call _getkey		;wait for a key to be pressed
   ret				;return from the program

undo:
   res 2,(iy+$23)		;turn off the interrupt program
   ld a,$3C				;the normal screen mem
   out (0),a			;change it to normal
   call _clrScrn		;clear the screen
   call _homeup			;cursor(x,y)=(1,1)
   ret					;return from the program

TEXT:	;just the text to be displayed (21 spaces per line)
	.db "F1+F2 = spd indicator"
	.db "F1+F2+F4 = spd down  "
	.db "F1+F2+F5 = spd up    "
	.db "F1+F4 = quick spd dwn"
	.db "F1+F5 = quick spd up "
	.db "Press any key...     ",0

int:
   push af			;when you run an interrupt, you can't change ANY registers
   push bc			;so you must push all the regesters you use
   push de
   push hl			;and pop (restore) them at the end
  ld hl,Speed
  ld a,%00111111	;key port mask
  out (1),a			;send it to the key port
  in a,(1)			;get it back
  cp %11100111
  jr z,SHOW_SPEED	;user holding F1 and F2, display the speed
  cp %11101110
  jr z,UP			;user holding F1 and F5, speed it up
  cp %11101101
  jr z,DOWN			;user holding F1 and F4, decrease the speed
  cp %11101011
  jr z,NORM			;user holding F1 and F3, restore the normal screen
cont:
  ld hl,var			;var is the counter which creates the delay
  dec (hl)			;it's decreased by one everytime the interupt occurs
  jr nz,done		;if it's not 0, quit the interrupt
  ld a,(Speed)		;if the counter has counted down
  ld (hl),a			;restore it with the speed
inverse_screen:
  ld de,$F600		;point de to the top of the new screen memory
  ld hl,$FC00		;point hl to the normal screen memory
  ld bc,$0004		;b will be 128 and c will be 8 (we will loop 
Loop:
  ld a,$FF		;%11111111 in binary, when this is xor'd to the screen
  xor (hl)		;it will inverse the data of the normal screen
  ld (de),a		;load the inversed data into the new display
  inc hl		;increase hl to the next byte in the screen data
  inc de		;increase de to point to the next byte in the new screen data
  djnz Loop		;dec b and if it's not zero, Loop
				;b will be decreased from 0 to 255 ($FF)
  dec c			;decrement c 
  jr nz,Loop	;loop if it's not 0
done:
   pop hl		;restore our variables in the opposide order we pushed them
   pop de
   pop bc
   pop af
   ret			;return from the interrupt

NORM:
  ld a,$3C		;restore the normal screen for those games that disable the int's
  out (0),a		;(press F1+F2 to go back to the inversed screen if the int is still
  jr done		;running)

UP:
  ld a,(hl)		;a=value of speed
  dec a			;subtract 1
  cp 2			;check if it's 2 (any faster and the calc crashes
				;because the interrupt is called before it finishes, then
				;you can never get out of it)
Checked0:
  jr z,cont		;if it is, don't save it
  ld (hl),a		;save speed
  jr cont		;continue

DOWN:
  ld a,(hl)		;a=value of speed
  inc a			;increase by 1
				;z is set if it looped passed 255 so we don't need a cp here
  jr Checked0	;save some space by jumping up to what was be here & is elsewhere

SHOW_SPEED:
  ld a,$36			;$36+$0C*$100 = F600 (the new video mem location)
  out (0),a			;make sure the calc's video mem is set to display this
  ld hl,$F9D0	;2nd to last line of new screen mem
  ld b,48		;16 bytes per line=3 lines to clear
Clear_Loop:
  ld (hl),0		;clear it
  inc hl		;next byte
  djnz Clear_Loop
  ld hl,$F9FE
  ld (hl),$FF
  ld hl,$F9CF	;1 byte before the last line of new screen mem
  ld a,(Speed)
  ld b,a
  ld a,1
  sub b 		;subtracting 133 from the speed puts a between 1 and 127...
  ld b,a		;...since there are only 128 pixels per line
  xor a			;0->a
  rr b			;divide b by 16, remainder -> a ...
  rra
  rr b
  rra
  rr b
  rra
  rlca
  rlca
  rlca			;... a is now the remainder
  inc b			;incase b=0, inc it (b=# of times to increase hl)
incHL:
  inc hl		;point hl to the right byte
  djnz incHL
  ld b,a		;a is the remainder (# of bits to move forward
  inc b			;inc it incase it's 0
  ld a,%00000001
rrcaLOOP:
  rrca			;rotate a right circular to put the 1 in the right bit
  djnz rrcaLOOP
  xor (hl)		;inverse the bit representing the speed
  ld (hl),a		;load the byte containing the inversed bit into screen mem
  ld de,16
  add hl,de
  xor (hl)
  ld (hl),a
  ld b,20		;delay counter 1
DELAY:
  push bc		;hide it
  ld b,255		;delay counter 2
d2: djnz d2		;delay (dec b until b=0)
  pop bc		;restore counter 1
  djnz DELAY	;dec b and repeat until  b=0

  ld a,%00111111	;key port mask
  out (1),a			;send it to the key port
  in a,(1)			;get it back
  cp %11100110
  jr z,SS_UP		;you are holding F1, F2 and F5, so we will speed it up
  cp %11100101
  jr z,SS_DOWN		;you are holding F1, F2 and F4, so we will decrease the speed
  cp %11100111
  jr nz,done		;if you aren't holding F1 and F2, exit this pause
  jr SHOW_SPEED		;loop (pause) 

SS_UP:
  ld a,(Speed)		;hl points to value of speed
  dec a				;subtract 1
  cp 2				;check if it's 2
Checked:
  jr z,SHOW_SPEED	;if it is, don't save it
  ld (Speed),a		;save speed
  jr SHOW_SPEED		;continue (loop until the keys are released)

SS_DOWN:
  ld a,(Speed)	;do the same stuff as the up routine...
  inc a			;only increase instead of decreasing...
				;z is set if it looped passed 255 so we don't need a cp here
  jr Checked	;save some space by jumping up to what was be here & is elsewhere


var equ $D3C5-2		;locate this variable 2 bytes from the end of the int mem
  .db 0
Speed equ $D3C5-1	;locate this variable 2 bytes from the end of the int mem
  .db 0				;to avoid the checksum byte
int_end:

.end		;All DONE!

